/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <dlfcn.h>
#include <assert.h>
#include <errno.h>
#include <string.h>
#include <fcntl.h>                          /* fcntl() */
#include "initng.h"
#include "initng_service_cache.h"
#include "initng_load_module.h"
#include "initng_toolbox.h"
#include "initng_global.h"
#include "initng_signal.h"

#include "initng_plugin_callers.h"
#include "initng_plugin_type.h"

/* P L U G I N    C A L L E R S */

/* if plugin returns <0 , it will directly stop, returning the value */
/* if plugin returns 0  , it will continue with next plugin */
/* if plugin returns 1  , it will be increment the return value, and take next plugin */
/* if plugin returns >1 , it will directly stop, return the value */
int initng_plugin_call(e_ptype type, void *data_args)
{
    int tmp = 0;
    int to_ret = 0;
    s_call *current = g.call_db;

    while (current)
    {

        if (current->ptype == type)
        {
            tmp = (*current->c.hook) (data_args);
            if (tmp < 0)
            {                               /* -1 */
                F_("Plugin type %i did return: %i, wont go for next plugin.",
                   type, tmp);
                return (tmp);
            }
            if (tmp > 1)
            {                               /* +2 */
                F_("Plugin type %i did return: %i, wont go for next plugin.",
                   type, tmp);
                return (tmp);
            }
            if (tmp == 1)
                to_ret = 1;
        }



        current = current->next;
    }
    return (to_ret);
}

/* this function returns first pointer got */
void *initng_plugin_pcall(e_ptype type, void *data_args)
{
    void *data_ret = NULL;

    s_call *current = g.call_db;

    while (current)
    {
        if (current->ptype == type)
        {
            if ((data_ret = (*current->c.phook) (data_args)))
            {
                return (data_ret);
            }
        }
        current = current->next;
    }
    return (NULL);
}

int initng_plugin_handle_killed(active_h * s, process_h * p)
{
    s_call *current = g.call_db;

    while (current)
    {
        if (current->ptype == HANDLE_KILLED)
        {
            if (((*current->c.handle_killed) (s, p)) == TRUE)
                return (TRUE);
        }
        current = current->next;
    }
    return (FALSE);
}


int initng_plugin_after_fork(active_h * s, process_h * p)
{
    int to_ret;
    s_call *current = g.call_db;

    assert(s);
    assert(p);
    D_("FORK IS DONE (%s)\n", s->name);


    while (current)
    {
        if (current->ptype == A_FORK)
        {
            if ((to_ret = (*current->c.af_launcher) (s, p)) == FALSE)
            {
                F_("Some after_fork plugin failed!\n");
                return (FALSE);
            }
        }
        current = current->next;
    }
    return (TRUE);
}


/* called when system state has changed. */
void initng_load_module_system_changed(h_sys_state state)
{
    s_call *current = g.call_db;
    s_call *next;

    while (current)
    {
        next = current->next;
        if (current->ptype == SWATCHERS && current->c.swatcher)
            (*current->c.swatcher) (state);
        current = next;
    }
    return;
}
